using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using MVVMDemo.Model;

namespace MVVMDemo.ViewModel
{
  public class PersonViewModel : ViewModelBase
  {
    
    private Person _person;
    private StringViewModel _lastname;
    private StringViewModel _firstname;
    private DateViewModel _birthdate;
    private bool _isNew;

    // 
    // Konstruktor:
    //
    public PersonViewModel(Person person)
    {
      if (person != null)
      {
        _person = person;
      }
      else
      {
        _person = new Person();
        _isNew = true;
      }
      _lastname = new StringViewModel(_person.Lastname);
      _firstname = new StringViewModel(_person.Firstname);
      DateTime? date = !_isNew ? (DateTime?)_person.Birthdate : null;
      _birthdate = new DateViewModel(date);

      _lastname.PropertyChanged += PropagateHasChangesAndHasError;
      _firstname.PropertyChanged += PropagateHasChangesAndHasError;
      _birthdate.PropertyChanged += PropagateHasChangesAndHasError;

      this.PropertyChanged += delegate(object sender, PropertyChangedEventArgs e)
      {
        // Wenn sich HasChanges ndert sich auch der HasChangesMarker
        if (e.PropertyName == "HasChanges") RaisePropertyChanged("HasChangesMarker");
      };

    }

    public string HasChangesMarker
    {
      get
      {
        if (this.HasChanges) return "*";
        return string.Empty;
      }
    }

    internal Person Person
    {
      get { return _person; }
    }

    public StringViewModel Lastname
    {
      get { return _lastname; }
    }

    public StringViewModel Firstname
    {
      get { return _firstname; }
    }

    public DateViewModel Birthdate
    {
      get { return _birthdate; }
    }

    public bool IsNew
    {
      get { return _isNew; }
      internal set
      {
        if (value != _isNew)
        {
          _isNew = value;
          RaisePropertyChanged("IsNew");
        }
      }
    }

    public void SendChangesToModel()
    {
      if (_isNew && _person == null)
      {
        _person = new Person();
      }
      WriteToModel();
    }

    public override void UndoChanges()
    {
      if (!_isNew)
      {
        ReadFromModel();
        Lastname.AcceptChanges();
        Firstname.AcceptChanges();
        Birthdate.AcceptChanges();
      }
    }

    public override void AcceptChanges()
    {
      ReadFromModel(); // zur Sicherheit nochmals lesen
      Lastname.AcceptChanges();
      Firstname.AcceptChanges();
      Birthdate.AcceptChanges();
      IsNew = false;
      HasChanges = false;
    }

    private void ReadFromModel()
    {
      _lastname.Value = _person.Lastname;
      _firstname.Value = _person.Firstname;
      if (_person.Birthdate != default(DateTime))
        _birthdate.Value = _person.Birthdate.Date;
    }

    private void WriteToModel()
    {
      _person.Lastname = _lastname.Value;
      _person.Firstname = _firstname.Value;
      _person.Birthdate = _birthdate.Value;
    }

    private void PropagateHasChangesAndHasError(object sender, PropertyChangedEventArgs e)
    {
      if (e.PropertyName == "HasChanges")
      {
        HasChanges = _lastname.HasChanges || _firstname.HasChanges || _birthdate.HasChanges;
      }
      else if (e.PropertyName == "HasError")
      {
        HasError = _lastname.HasError || _firstname.HasError || _birthdate.HasError;
      }
    }

  }
}
