using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;

namespace MVVMDemo.ViewModel
{

  /// <summary>
  /// ViewModel fr einen String-Wert.
  /// Merkt sich den Original-Wert fr das UndoChanges().
  /// </summary>
  public class StringViewModel : ViewModelBase, IComparable
  {
    private string _value;
    private string _originalValue;

    public StringViewModel(string value)
    {
      _value = value;
      _originalValue = value;
    }

    // Der eigentliche Wert.
    public string Value
    {
      get { return _value; }
      set
      {
        if (value != _value)
        {
          _value = value;
          RaisePropertyChanged("Value");
          HasChanges = _value != _originalValue && !(string.IsNullOrEmpty(_value) && string.IsNullOrEmpty(_originalValue));
        }
      }
    }

    public override void AcceptChanges()
    {
      _originalValue = _value;
      HasChanges = false;
      HasError = false;
    }

    public override void UndoChanges()
    {
      Value = _originalValue;
      HasError = false;
    }

    // Muss implementiert werden, damit die Sortierung im CollectionView funktioniert.
    public int CompareTo(object obj)
    {
      if (obj == null) return 1;
      if(object.ReferenceEquals(this, obj)) return 0;
      StringViewModel other = obj as StringViewModel;
      return string.Compare(this._value, other._value);
    }

    public override string ToString()
    {
      return _value;
    }

  }

}
