using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;

namespace MVVMDemo.ViewModel
{
  // <summary>
  // Basisklasse fr alle ViewModel Klassen
  // </summary>
  public abstract class ViewModelBase : INotifyPropertyChanged
  {
    private bool _hasChanges;
    private bool _hasError;
    private string _errorMessage;

    #region INotifyPropertyChanged Implementation

    // Das im INotifyPropertyChanged Intrface definierte Event
    public event PropertyChangedEventHandler PropertyChanged;

    // bliche virtuelle Methode, durch die Sub-Klassen das Auslsen des Events
    // berschreiben knnen
    protected virtual void OnPropertyChanged(PropertyChangedEventArgs e)
    {
      if (PropertyChanged != null)
      {
        PropertyChanged(this, e);
      }
    }

    // Hilfsmethode, um das PropertyChanged Event auszulsen 
    protected void RaisePropertyChanged(string propertyName)
    {
      PropertyChangedEventArgs e = new PropertyChangedEventArgs(propertyName);
      OnPropertyChanged(e);
    }
    #endregion

    // Gibts es nderungen auf diesem ViewModel Objekt?
    // Sollte auch true liefern, wenn in der Objekt-Hierarchie untergeordnete 
    // ViewModel-Objekte Changes haben
    public bool HasChanges
    {
      get
      {
        return _hasChanges;
      }
      protected set
      {
        if (value != _hasChanges)
        {
          _hasChanges = value;
          RaisePropertyChanged("HasChanges");
        }
      }
    }

    // Logik von Sub-Klassen um die alle nderungen zu bernehmen 
    // und HasChanges zurckzusetzen.
    public abstract void AcceptChanges();

    // Logik von Sub-Klassen um die alle nderungen rckgngig zu machen 
    // und HasChanges zurckzusetzen.
    public abstract void UndoChanges();

    // Gibts es Fehler diesem ViewModel Objekt?
    // Sollte auch true liefern, wenn in der Objekt-Hierarchie untergeordnete 
    // ViewModel-Objekte Fehler haben
    public bool HasError
    {
      get { return _hasError; }
      protected set
      {
        if (value != _hasError)
        {
          _hasError = value;
          if (!_hasError) ErrorMessage = null;
          RaisePropertyChanged("HasError");
        }
      }
    }

    // Fehler Message
    public string ErrorMessage
    {
      get
      {
        if (!HasError) return string.Empty;
        return _errorMessage;
      }
      protected set
      {
        if (value != _errorMessage)
        {
          _errorMessage = value;
          if (!string.IsNullOrEmpty(_errorMessage)) HasError = true;
          RaisePropertyChanged("ErrorMessage");
        }
      }
    }

  }
}
