using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using MVVMDemo.Model;
using MVVMDemo.ViewModel;

namespace PersonDataProvider
{
  /// <summary>
  /// Sehr einfache Implementation von IPersonDataProvider.
  /// </summary>
  public class SimplePersonDataProvider : IPersonDataProvider
  {
    private const string c_path = "Persons.dat";
    private List<Person> _persons;
    private BinaryFormatter _formatter = new BinaryFormatter();


    public List<Person> GetPersons()
    {
      if (_persons == null) LoadPersons();
      return _persons;
    }

    public void InsertPerson(Person person)
    {
      _persons.Add(person);
      Debug.WriteLine("Insert: " + person.ToString());
      WritePersons();
    }

    public void UpdatePerson(Person person)
    {
      WritePersons();
      Debug.WriteLine("Update: " + person.ToString());
    }

    public void DeletePerson(Person person)
    {
      _persons.Remove(person);
      WritePersons();
      Debug.WriteLine("Delete: " + person.ToString());
    }

    private void LoadPersons()
    {
      if (!File.Exists(c_path))
      {
        _persons = new List<Person>();
      }
      else
      {
        using (FileStream fs = File.Open(c_path, FileMode.Open))
        {
          _persons = (List<Person>)_formatter.Deserialize(fs);
        }
      }
    }

    private void WritePersons()
    {
      using (FileStream fs = File.Open(c_path, FileMode.Create))
      {
        _formatter.Serialize(fs, _persons);
      }
    }

  }
}
