using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using NUnit.Framework;
using MVVMDemo.Model;
using MVVMDemo.ViewModel;

namespace UnitTests
{

  /// <summary>
  /// Test Data-Provider
  /// </summary>
  public class TestDataProvider : IPersonDataProvider
  {

    public int Updates, Inserts, Deletes;

    public List<Person> GetPersons()
    {
      List<Person> list = new List<Person>();
      list.Add(new Person("A", "Y", new DateTime(2000, 1, 1)));
      list.Add(new Person("B", "X", new DateTime(2000, 12, 31)));
      return list;
    }

    public void InsertPerson(Person person)
    {
      Inserts++;
    }

    public void UpdatePerson(Person person)
    {
      Updates++;
    }

    public void DeletePerson(Person person)
    {
      Deletes++;
    }

  }

  /// <summary>
  /// Beispiel-Tests fr das MainViewModel
  /// </summary>
  [TestFixture]
  public class ViewModelTests
  {
    TestDataProvider dataProvider;
    MainViewModel mainVM;

    [SetUp]
    public void InitTest()
    {
      dataProvider = new TestDataProvider();
      mainVM = new MainViewModel(dataProvider);
      mainVM.LoadPersons();
    }

    [Test]
    public void TestSorting()
    {
      // Per Default muss nach 'Firstname' sortiert sein:
      mainVM.PersonsView.MoveCurrentToFirst();
      PersonViewModel personVM = (PersonViewModel)mainVM.PersonsView.CurrentItem;
      Assert.AreEqual("A", personVM.Firstname.Value);

      // Nach 'Lastname' sortieren:
      mainVM.SortByProperty = "Lastname";
      mainVM.PersonsView.MoveCurrentToFirst();
      personVM = (PersonViewModel)mainVM.PersonsView.CurrentItem;
      Assert.AreEqual("X", personVM.Lastname.Value);
    }

    [Test]
    public void TestDelete()
    {
      
      bool userAsked = false;

      // Antwort auf die MessageBox simulieren:
      mainVM.UserFeedbackRequired += delegate(object sender, UserFeedbackEventArgs e)
      {
        userAsked = true;
        e.Result = MessageBoxResult.Yes;
      };

      // Person lschen:
      mainVM.PersonsView.MoveCurrentToFirst();
      mainVM.HandleDeletePersonExecuted(null, null); // Argumente werden nicht verwendet

      Assert.IsTrue(userAsked);
      Assert.IsTrue(mainVM.HasChanges);
      Assert.AreEqual(1, mainVM.PersonsView.Count);

      // nderung speichern:
      mainVM.HandleSavePersonsExecuted(null, null);
      Assert.AreEqual(1, dataProvider.Deletes);
      Assert.IsFalse(mainVM.HasChanges);
      Assert.AreEqual(1, mainVM.PersonsView.Count);
    }

  }
}
